@@ -183,7 +183,7 @@ class SampleBatch(dict):
             >>> print(SampleBatch.concat_samples([b1, b2]))
             {"a": np.array([1, 2, 3]), "b": np.array([10, 11, 12])}
         """
-        if isinstance(samples[0], MultiAgentBatch):
+        if any(isinstance(s, MultiAgentBatch) for s in samples):
             return MultiAgentBatch.concat_samples(samples)
         concatd_seq_lens = []
         concat_samples = []
@@ -364,6 +364,9 @@ class SampleBatch(dict):
             for i, p in enumerate(path):
                 if i == len(path) - 1:
                     curr[p] = value[permutation]
+                # Translate into list (tuples are immutable).
+                if isinstance(curr[p], tuple):
+                    curr[p] = list(curr[p])
                 curr = curr[p]
 
         tree.map_structure_with_path(_permutate_in_place, self)
@@ -688,9 +691,6 @@ class SampleBatch(dict):
         if isinstance(key, slice):
             return self._slice(key)
 
-        if not hasattr(self, key) and key in self:
-            self.accessed_keys.add(key)
-
         # Backward compatibility for when "input-dicts" were used.
         if key == "is_training":
             if log_once("SampleBatch['is_training']"):
@@ -700,6 +700,9 @@ class SampleBatch(dict):
                     error=False)
             return self.is_training
 
+        if not hasattr(self, key) and key in self:
+            self.accessed_keys.add(key)
+
         value = dict.__getitem__(self, key)
         if self.get_interceptor is not None:
             if key not in self.intercepted_values:
@@ -721,6 +724,16 @@ class SampleBatch(dict):
             dict.__setitem__(self, key, item)
             return
 
+        # Backward compatibility for when "input-dicts" were used.
+        if key == "is_training":
+            if log_once("SampleBatch['is_training']"):
+                deprecation_warning(
+                    old="SampleBatch['is_training']",
+                    new="SampleBatch.is_training",
+                    error=False)
+            self.is_training = item
+            return
+
         if key not in self:
             self.added_keys.add(key)
 
@@ -1158,7 +1171,12 @@ class MultiAgentBatch:
         policy_batches = collections.defaultdict(list)
         env_steps = 0
         for s in samples:
+            # Some batches in `samples` are not MultiAgentBatch.
             if not isinstance(s, MultiAgentBatch):
+                # If empty SampleBatch: ok (just ignore).
+                if isinstance(s, SampleBatch) and len(s) <= 0:
+                    continue
+                # Otherwise: Error.
                 raise ValueError(
                     "`MultiAgentBatch.concat_samples()` can only concat "
                     "MultiAgentBatch types, not {}!".format(type(s).__name__))
